/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gridCubeSF.h"

#include "../variousEntities/prism.h"


GridCubeSF::GridCubeSF(int size, std::vector<int> colors, float spheresRadius, float cylinderRadius, float interval):
	GridCube (size, colors, interval, cylinderRadius) {
	
	mainEntitiesIndex = 0;
	torusMesh = new TorusMesh(cylinderRadius * 6., 12, cylinderRadius * 4., 8, false);
	scene->addMeshInTheSet(torusMesh, materials[1]->getNumberOfTextures());		
	EntityCubeTab *tabTorus = new EntityCubeTab(size, interval, true,
							 					Vector3(1., 0., 0.), Vector3(0.,1.,0.), Vector3(0.,0.,1.),
							 					Vector3(-interval * ((float) size - 1.) / 2., -interval * ((float) size - 1.) / 2., -interval * ((float) size - 1.) / 2.),
												Vector3(0.,1.,0.), 0.);
		
	addChild(tabTorus, mainEntitiesIndex);

	cylinderAxis = new CylinderMesh (((float) mSize - 1.) * mInterval, 1 , cylinderRadius, 5, false);
	scene->addMeshInTheSet(cylinderAxis, materials[1]->getNumberOfTextures());		
		
		
	CylinderMesh *verticalCylinderMesh = new CylinderMesh (((float) mSize - 0.5) * mInterval, 1 , cylinderRadius * 2., 8, BASE_DOME, BASE_NONE, 3, false);	
	verticalRodsIndex = 1;	
		
	EntityPlane *verticalRods =	new EntityPlane (size, interval, true, materials[0], verticalCylinderMesh,
												Vector3(0.,1.,0.), Vector3(0.,0.,1.),
												Vector3(-interval * ((float) size - 1.) / 2., 0., -interval * ((float) size - 1.) / 2.),
												Vector3(0.,0.,1.), -90.);
	verticalRods->setClickable();
	addChild(verticalRods, verticalRodsIndex);
		
		
	baseIndex = 2;	
	PrismWithPosition *prism = new PrismWithPosition(4, 0.8, 1, ((float) size ) / 2. * interval, 1, 1.2, 3,
													 0., BASE_FLAT, BASE_FLAT, false, materials[0], materials[0], materials[0]);
	prism->setPosition(Vector3(0., -((float) size - 0.5) / 2. * interval,0.));	
	addChild(prism, baseIndex); 
		
}

GridCubeSF::~GridCubeSF() {

}



void GridCubeSF::reinit() {
	Entity *tabSpheres = children[mainEntitiesIndex];
	tabSpheres->removeAllChildren();
	
	EntityPlane *axis = dynamic_cast <EntityPlane*> (children[verticalRodsIndex]);	
	axis->setValueAndMaterialForAllEntities(0, materials[0]);	
	
	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		int key = (*it).first;
		if(key<0) removeChild(key);
	}

}


int GridCubeSF::hitToPosition(int hit, int player) {
	int i,j,k, position;
	
	i = hit / mSize;
	k = hit % mSize;
	
	j = 0;
	position = i*mSize*mSize + k;
	
	EntityCubeTab *tab = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);			
	
	while((j<mSize)&&(tab->isAChild(position))) {
		j++;
		position += mSize;
	}
	
	if(j==mSize) {
		return -1;
	} else {
  		return position;
	}
}


void GridCubeSF::insertEntity(int position, int player) {
	EntityCubeTab *mainEntities = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);	
	
	int i = position / (mSize*mSize);
	int r = position % (mSize*mSize);
	int j = r / mSize;
	int k = r % mSize;
	
	mainEntities->insertChild(materials[2*player-1], torusMesh, player, i, j, k);		
}


int GridCubeSF::updateAxis(int player, int i, int j, int k) {
	int res = 0;
	EntityCubeTab *tab = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);	
	EntityPlane *axis;	


	if(tab->testAxis(1, j, k, player) == mSize){
		EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderAxis, true,
														 Vector3(0.,((float) j - ((float) mSize - 1.)/2.) * mInterval, ((float) k - ((float) mSize - 1.)/2.) * mInterval), 
														 Vector3(0., 0., 1.), 0.);
		addChild(cyl);		
		res ++;
	}

	if(tab->testAxis(2, k, i, player) == mSize){
		axis = dynamic_cast <EntityPlane*> (children[verticalRodsIndex]);	
		axis->setValueAndMaterial(player, materials[2*player-1], i*mSize+k);
		res ++;
	}
	
	if(tab->testAxis(3, i, j, player) == mSize){
		EntityWithPosition *cyl = new EntityWithPosition(materials[2*player-1], cylinderAxis, true,
														 Vector3(((float) i - ((float) mSize - 1.)/2.) * mInterval,((float) j - ((float) mSize - 1.)/2.) * mInterval, 0.), 
														 Vector3(0.,1.,0.), 90.);
		addChild(cyl);	
		res ++;
	}
	return res;
}

